;License Information------------------------------------
;Copyright (C) 2010 Andrew Calcutt
;This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; Version 2 of the License.
;This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
;You should have received a copy of the GNU General Public License along with this program; if not, visit http://www.gnu.org/licenses/gpl-2.0.html
;--------------------------------------------------------
#include <ButtonConstants.au3>
#include <EditConstants.au3>
#include <GUIConstantsEx.au3>
#include <StaticConstants.au3>
#include <WindowsConstants.au3>

Const $EarthRadius = 6378137 ;meters
Const $PI = 3.14159265358979

$settings = "rgsettings.ini"
$defLat = IniRead($settings, "rectangle", "NWLatitude", "N 0.0000000")
$defLon = IniRead($settings, "rectangle", "NWLongitude", "E 0.0000000")
$defHeight = IniRead($settings, "rectangle", "Height", "")
$defWidth = IniRead($settings, "rectangle", "Width", "")
$defHtype = IniRead($settings, "rectangle", "Htype", "kilometer(s)")
$defWtype = IniRead($settings, "rectangle", "Wtype", "kilometer(s)")


$RGGUI = GUICreate("Rectangle Generator v2 - By Andrew Calcutt", 467, 195)
GUICtrlCreateGroup("Rectangle Details", 8, 8, 449, 177)
GUICtrlCreateLabel("Latitude of NW Point", 23, 33, 103, 17)
$NWLat = GUICtrlCreateInput($defLat, 23, 48, 177, 21)
GUICtrlCreateLabel("Latitude of NW Point", 245, 33, 103, 17)
$NWLon = GUICtrlCreateInput($defLon, 245, 48, 177, 21)
GUICtrlCreateLabel("Width (to the east)", 23, 88, 91, 17)
$rWidth = GUICtrlCreateInput($defWidth, 23, 104, 97, 21)
$rWtype = GUICtrlCreateCombo("kilometer(s)", 128, 104, 75, 25)
GUICtrlSetData(-1, "mile(s)|degree(s)", $defWtype)
GUICtrlCreateLabel("Height (to the south)", 243, 92, 100, 17)
$rHeight = GUICtrlCreateInput($defHeight, 243, 108, 97, 21)
$rHtype = GUICtrlCreateCombo("kilometer(s)", 348, 108, 75, 25)
GUICtrlSetData(-1, "mile(s)|degree(s)", $defHtype)
$But_Exit = GUICtrlCreateButton("Exit", 56, 143, 129, 25, $WS_GROUP)
$But_GenKML = GUICtrlCreateButton("Generate KML", 270, 143, 129, 25, $WS_GROUP)
GUICtrlCreateGroup("", -99, -99, 1, 1)
GUISetState(@SW_SHOW)

While 1
	$nMsg = GUIGetMsg()
	Switch $nMsg
		Case $GUI_EVENT_CLOSE
			_Exit()
		Case $But_Exit
			_Exit()
		Case $But_GenKML
			$ldatetimestamp = StringFormat("%04i", @YEAR) & '-' & StringFormat("%02i", @MON) & '-' & StringFormat("%02i", @MDAY) & ' ' & @HOUR & '-' & @MIN & '-' & @SEC
			$kml = FileSaveDialog("Google Earth Output File", @ScriptDir, 'Google Earth (*.kml)', '', $ldatetimestamp & '.kml')
			If Not @error Then
				$StartLat = _Format_GPS_All_to_DMM(GUICtrlRead($NWLat), "N", "S")
				$StartLon = _Format_GPS_All_to_DMM(GUICtrlRead($NWLon), "E", "W")

				If GUICtrlRead($rWtype) = "kilometer(s)" Then
					$DistHor = GUICtrlRead($rWidth) * 1000;km to meters
					ConsoleWrite("kilometer" & @CRLF)
				ElseIf GUICtrlRead($rWtype) = "mile(s)" Then
					$DistHor = _MilesToMeters(GUICtrlRead($rWidth))
					ConsoleWrite("Miles" & @CRLF)
				ElseIf GUICtrlRead($rWtype) = "degree(s)"  Then
					$DistHor = _LongitudeDegToM($StartLat, GUICtrlRead($rWidth))
					ConsoleWrite("degree" & @CRLF)
				EndIf
				ConsoleWrite(GUICtrlRead($rWtype) & ' - ' & $DistHor & @CRLF)

				If GUICtrlRead($rHtype) = "kilometer(s)" Then
					$DistVert = GUICtrlRead($rHeight) * 1000;km to meters
					ConsoleWrite("kilometer" & @CRLF)
				ElseIf GUICtrlRead($rHtype) = "mile(s)" Then
					$DistVert = _MilesToMeters(GUICtrlRead($rHeight))
					ConsoleWrite("Miles" & @CRLF)
				ElseIf GUICtrlRead($rHtype) = "degree(s)"  Then
					$DistVert = _LatitudeDegToM(GUICtrlRead($rHeight))
					ConsoleWrite("degree" & @CRLF)
				EndIf
				ConsoleWrite(GUICtrlRead($rHtype) & ' - ' & $DistVert & @CRLF)


				$LeftLat = _DestLat($StartLat, "90", $DistHor)
				$LeftLon = _DestLon($StartLat, $StartLon, $LeftLat, "90", $DistHor)


				$BotLat = _DestLat($StartLat, "180", $DistVert)
				$BotLon = _DestLon($StartLat, $StartLon, $BotLat, "180", $DistVert)




				$kmlfile = '<?xml version="1.0" encoding="UTF-8"?>' & @CRLF _
						 & '<kml xmlns="http://www.opengis.net/kml/2.2" xmlns:gx="http://www.google.com/kml/ext/2.2" xmlns:kml="http://www.opengis.net/kml/2.2" xmlns:atom="http://www.w3.org/2005/Atom">' & @CRLF _
						 & '<Document>' & @CRLF _
						 & '<description>Rectangle Generator - By Andrew Calcutt</description>' & @CRLF _
						 & '<Style id="Location">' & @CRLF _
						 & '<LineStyle>' & @CRLF _
						 & '<color>7F0000FF</color>' & @CRLF _
						 & '<width>4</width>' & @CRLF _
						 & '</LineStyle>' & @CRLF _
						 & '</Style>' & @CRLF _
						 & '<Placemark>' & @CRLF _
						 & '<styleUrl>#Location</styleUrl>' & @CRLF _
						 & '<LineString>' & @CRLF _
						 & '<extrude>1</extrude>' & @CRLF _
						 & '<tessellate>1</tessellate>' & @CRLF _
						 & '<coordinates>' & @CRLF _
						 & StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($StartLon), 'W', '-'), 'E', ''), ' ', '') & ',' & StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($StartLat), 'S', '-'), 'N', ''), ' ', '') & ',0' & @CRLF _
						 & StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($LeftLon), 'W', '-'), 'E', ''), ' ', '') & ',' & StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($LeftLat), 'S', '-'), 'N', ''), ' ', '') & ',0' & @CRLF _
						 & StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($LeftLon), 'W', '-'), 'E', ''), ' ', '') & ',' & StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($BotLat), 'S', '-'), 'N', ''), ' ', '') & ',0' & @CRLF _
						 & StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($BotLon), 'W', '-'), 'E', ''), ' ', '') & ',' & StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($BotLat), 'S', '-'), 'N', ''), ' ', '') & ',0' & @CRLF _
						 & StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($StartLon), 'W', '-'), 'E', ''), ' ', '') & ',' & StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($StartLat), 'S', '-'), 'N', ''), ' ', '') & ',0' & @CRLF _
						 & '</coordinates>' & @CRLF _
						 & '</LineString>' & @CRLF _
						 & '</Placemark>' & @CRLF _
						 & '</Document>' & @CRLF _
						 & '</kml>' & @CRLF

				FileDelete($kml)
				$filewrite = FileWrite($kml, $kmlfile)
				If $filewrite = 0 Then
					MsgBox(0, "Error", "Error writing file")
				Else
					MsgBox(0, "Done", "File saved as: " & $kml)
				EndIf
			EndIf

	EndSwitch
WEnd

Func _Exit()
	IniWrite($settings, "rectangle", "NWLatitude", GUICtrlRead($NWLat))
	IniWrite($settings, "rectangle", "NWLongitude", GUICtrlRead($NWLon))
	IniWrite($settings, "rectangle", "Height", GUICtrlRead($rHeight))
	IniWrite($settings, "rectangle", "Width", GUICtrlRead($rWidth))
	Exit
EndFunc

Func _DestLat($Lat1, $Brng1, $Dist1)
	$Lat1 = _deg2rad(StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($Lat1), " ", ""), "N", ""), "S", "-"))
	$Brng1 = _deg2rad($Brng1)
	Return (_Format_GPS_All_to_DMM(_rad2deg(ASin(Sin($Lat1) * Cos($Dist1 / $EarthRadius) + Cos($Lat1) * Sin($Dist1 / $EarthRadius) * Cos($Brng1))), "N", "S"));Return destination dmm latitude
EndFunc   ;==>_DestLat

Func _DestLon($Lat1, $Lon1, $Lat2, $Brng1, $Dist1)
	$Lat1 = _deg2rad(StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($Lat1), " ", ""), "N", ""), "S", "-"))
	$Lon1 = _deg2rad(StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($Lon1), " ", ""), "E", ""), "W", "-"))
	$Lat2 = _deg2rad(StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($Lat2), " ", ""), "N", ""), "S", "-"))
	$Brng1 = _deg2rad($Brng1)
	Return (_Format_GPS_All_to_DMM(_rad2deg($Lon1 + _ATan2(Cos($Dist1 / $EarthRadius) - Sin($Lat1) * Sin($Lat2), Sin($Brng1) * Sin($Dist1 / $EarthRadius) * Cos($Lat1))), "E", "W"));Return destination dmm longitude
EndFunc   ;==>_DestLon

Func _Format_GPS_DMM_to_DDD($gps);converts gps position from ddmm.mmmm to dd.ddddddd
	$return = '0.0000000'
	$splitlatlon1 = StringSplit($gps, " ");Split N,S,E,W from data
	If $splitlatlon1[0] = 2 Then
		$splitlatlon2 = StringSplit($splitlatlon1[2], ".");Split dd from data
		$latlonleft = StringTrimRight($splitlatlon2[1], 2)
		$latlonright = (StringTrimLeft($splitlatlon2[1], StringLen($splitlatlon2[1]) - 2) & '.' & $splitlatlon2[2]) / 60
		$return = $splitlatlon1[1] & ' ' & StringFormat('%0.7f', $latlonleft + $latlonright);set return
	EndIf
	Return ($return)
EndFunc   ;==>_Format_GPS_DMM_to_DDD

Func _Format_GPS_All_to_DMM($gps, $PosChr, $NegChr);converts dd.ddddddd, 'dd mm' ss", or ddmm.mmmm to ddmm.mmmm
	Local $return = "N 0.0000"
	If StringInStr($gps, '-') Or StringInStr($gps, $NegChr) Then
		$gDir = $NegChr
	Else
		$gDir = $PosChr
	EndIf
	$gps = StringReplace(StringReplace(StringReplace(StringReplace(StringReplace($gps, " ", ""), "-", ""), "+", ""), $PosChr, ""), $NegChr, "")
	$splitlatlon2 = StringSplit($gps, ".")
	If $splitlatlon2[0] = 1 Then
		$DDSplit = StringSplit($gps, "")
		If $DDSplit[0] = 2 Then
			$DD = $DDSplit[1] * 100
			$MMSplit = StringSplit($DDSplit[2], "'")
			If $MMSplit[0] = 2 Then
				$MM = $MMSplit[1] + (StringReplace($MMSplit[2], '"', '') / 60)
				$return = $gDir & ' ' & StringFormat('%0.4f', $DD + $MM)
			EndIf
		EndIf
	ElseIf $splitlatlon2[0] = 2 Then
		If StringLen($splitlatlon2[2]) = 4 Then ;ddmm.mmmm to ddmm.mmmm
			$return = $gDir & ' ' & StringFormat('%0.4f', $gps)
		Else; dd.dddd to ddmm.mmmm
			$DD = $splitlatlon2[1] * 100
			$MM = ('.' & $splitlatlon2[2]) * 60 ;multiply remaining decimal by 60 to get mm.mmmm
			$return = $gDir & ' ' & StringFormat('%0.4f', $DD + $MM);Format data properly (ex. N ddmm.mmmm)
		EndIf
	EndIf
	Return ($return)
EndFunc   ;==>_Format_GPS_All_to_DMM

Func _deg2rad($Degree) ;convert degrees to radians
	$Degree = StringReplace(StringReplace(StringReplace(StringReplace(StringReplace($Degree, 'W', '-'), 'E', ''), 'S', '-'), 'N', ''), ' ', '')
	Return ($Degree * ($PI / 180))
EndFunc   ;==>_deg2rad

Func _rad2deg($radian) ;convert radians to degrees
	Return ($radian * (180 / $PI))
EndFunc   ;==>_rad2deg

Func _LongitudeDegToM($Lat1, $degrees);converts degress of longitude to meters (based on latitude)
	$Lat1 = StringReplace(StringReplace(StringReplace(_Format_GPS_DMM_to_DDD($Lat1), " ", ""), "N", ""), "S", "-")
	ConsoleWrite($Lat1 &  ' - ' & $degrees &  ' - ' & (cos($Lat1) * 111325) & @CRLF)
	Return((cos($Lat1) * 111325) * $degrees)
EndFunc

Func _LatitudeDegToM($degrees);converts degress of latitude to meters
	Return(111325 * $degrees)
EndFunc

Func _MilesToMeters($miles)
	Return($miles * 1609.3)
EndFunc

Func _ATan2($X, $y) ;ATan2 function, since autoit only has ATan
	Local Const $PI = 3.14159265358979
	If $y < 0 Then
		Return -_ATan2($X, -$y)
	ElseIf $X < 0 Then
		Return $PI - ATan(-$y / $X)
	ElseIf $X > 0 Then
		Return ATan($y / $X)
	ElseIf $y <> 0 Then
		Return $PI / 2
	Else
		SetError(1)
	EndIf
EndFunc   ;==>_ATan2


